// WARNING: ONLY ONE program can call FliteDeck functions.
// if second program wish call them, FliteDeck will return error code: FD_E_ALREADY_INITIALIZED

#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "FliteDeck.h"

void VIA_Sleep(int nSec)
{
	Sleep(nSec*1000);
}
/************************************************************************
		GPIO
************************************************************************/
void GpioExample()
{
	int i;
	TFDGpioInfo aGpioInfo;
	BYTE aValue;

	if ( !FDSdk_SupportGpio() ) {
		printf("The board does not support this function\n");
		return;
	}

	if ( FDGpio_GetInfo(&aGpioInfo)==FD_E_FAIL ) { return; }
	// set high for all of GPO
	for ( i=0; i<aGpioInfo.nGPONum; ++i ) {
		FDGpio_Write(aGpioInfo.auGPOList[i], 1);
	}
	// read all of GPI
	for ( i=0; i<aGpioInfo.nGPINum; ++i ) {
		FDGpio_Read(aGpioInfo.auGPIList[i], &aValue);
		printf("GPI%02d=%d\n", aGpioInfo.auGPIList[i], aValue);
	}
	// set low for all of GPO
	for ( i=0; i<aGpioInfo.nGPONum; ++i ) {
		FDGpio_Write(aGpioInfo.auGPOList[i], 0);
	}
	// read all of GPI
	for ( i=0; i<aGpioInfo.nGPINum; ++i ) {
		FDGpio_Read(aGpioInfo.auGPIList[i], &aValue);
		printf("GPI%02d=%d\n", aGpioInfo.auGPIList[i], aValue);
	}
}

/************************************************************************
		SMBUS
************************************************************************/
void SmbusExample()
{
	BYTE aValue;

	if ( !FDSdk_SupportSmbus() ) {
		printf("The board does not support this function\n");
		return;
	}
	// read SPD
	FDSmbus_ReadByte(0xa0, 0, &aValue);
	printf("smbus: slave address=0xa0, offset=0, value=0x%02x\n", aValue);
}

/************************************************************************
		LCD backlight
************************************************************************/
void LcdBltExample()
{
	BYTE aValue;

	if ( !FDSdk_SupportLcdBlt() ) {
		printf("The board does not support this function\n");
		return;
	}
	// set LcdBlt power on
	if ( FDLcdBlt_GetSwitch(&aValue) ) {
		if ( aValue==0 ) {
			FDLcdBlt_SetSwitch(1);
		}
	}
	// get brightness
	FDLcdBlt_GetBrightness(&aValue);
	printf("LcdBlt brightness=%d\n", aValue);
	// set brightness
	FDLcdBlt_SetBrightness(255);
}

/************************************************************************
  watchdog timer
************************************************************************/
void WdtExample()
{
	// the board supports watchdog timer or not
	if ( !FDSdk_SupportWdt() ) {
		printf("The board does not support this function\n");
		return;
	}

	printf("press CTRL+C to terminate this application.\nsystem will reboot about 30 seconds.\n");

	// set timeout, parameter = 1~60 sec
	// if the program does not call FDWdt_Refresh(),
	//   watchdog timer will reboot mainboard after 30 seconds
	FDWdt_SetTimer(30);
	// start the watchdog timer
	FDWdt_Start();

	// 1. use a endless loop to refresh watchdog timer
	// 2. or use thread to control watchdog timer is better
	while ( 1 ) {
		// 10 seconds to refresh the watchdog timer,
		//   the parameter of sleep() MUST BE less than the parameter of FDWdt_SetTimer()
		VIA_Sleep(10);
		FDWdt_Refresh();
	}

	// stop watchdog timer
	FDWdt_Stop();
}

/************************************************************************
		Hardware Monitor Information
************************************************************************/
void VoltageExample()
{
	DWORD nRetCode;
	double aVolt;
	int i;

	for ( i = 0; i<7; ++i ) {
		// voltage type = 1, 2, 4, 8, K
		nRetCode = FDHwmVoltage_Get((DWORD)(1<<i), & aVolt);
		switch ( nRetCode ) {
			case FD_E_FUNC_NOT_SUPPORT:
				printf("voltage %d is not supported on the board\n", i);
				break;
			case FD_E_HARDWARE_ERROR:
				printf("voltage %d is hardware error\n", i);
				break;
			default:
				printf("voltage %d is %7.2f V\n", i, aVolt);
				break;
		}
	}
}

void FanSpeedExample()
{
	DWORD nRetCode;
	DWORD aValue;

	nRetCode = FDHwmCpu_GetFanSpeed(&aValue);
	switch ( nRetCode ) {
		case FD_E_FUNC_NOT_SUPPORT:
			printf("The fan speed of CPU is not supported on the board\n");
			break;
		case FD_E_HARDWARE_ERROR:
			printf("The fan speed of CPU is hardware error\n");
			break;
		default:
			printf("The fan speed of CPU is %d PRM\n", aValue);
			break;
	}

	nRetCode = FDHwmSystem_GetFanSpeed(&aValue);
	switch ( nRetCode ) {
		case FD_E_FUNC_NOT_SUPPORT:
			printf("The fan speed of system is not supported on the board\n");
			break;
		case FD_E_HARDWARE_ERROR:
			printf("The fan speed of system is hardware error\n");
			break;
		default:
			printf("The fan speed of system is %d PRM\n", aValue);
			break;
	}
}

void TemperatureExample()
{
	DWORD nRetCode;
	double aValue;

	nRetCode = FDHwmCpu_GetTemperature(&aValue);
	switch ( nRetCode ) {
		case FD_E_FUNC_NOT_SUPPORT:
			printf("The temperature of CPU is not supported on the board\n");
			break;
		case FD_E_HARDWARE_ERROR:
			printf("The temperature of CPU is hardware error\n");
			break;
		default:
			printf("The temperature of CPU is %.1f degrees centigrade.\n", aValue);
			break;
	}

	nRetCode = FDHwmSystem_GetTemperature(&aValue);
	switch ( nRetCode ) {
		case FD_E_FUNC_NOT_SUPPORT:
			printf("The temperature of system is not supported on the board\n");
			break;
		case FD_E_HARDWARE_ERROR:
			printf("The temperature of system is hardware error\n");
			break;
		default:
			printf("The temperature of system is %.1f degrees centigrade.\n", aValue);
			break;
	}
}

void HwmExample()
{
	if ( !FDSdk_SupportHardwareMonitor() ) {
		printf("The board does not support this function\n");
		return;
	}

	VoltageExample();
	FanSpeedExample();
	TemperatureExample();
}

/************************************************************************
		Auto Power
************************************************************************/
void Add10Min(struct tm* pTm)
{
	time_t aTime;

	time(&aTime);
	*pTm = *localtime(&aTime);
	printf ( "current time and date: %s", asctime (pTm) );

	aTime += 600;
	*pTm = *localtime(&aTime);
	printf ( "after 10 minute: %s", asctime (pTm) );
}
void AutoPowerExample(BOOL nEnable)
{
	struct tm aTm;
	BYTE nDay = 0; // everyday
	BYTE nHour = 0;
	BYTE nMinute = 0;
	BYTE nSecond = 0;

	// the board supports auto power or not
	if ( !FDSdk_SupportAutoPower() ) {
		printf("The board does not support this function\n");
		return;
	}

	printf("This function is supported on a special BIOS!\n");

	if ( nEnable ) { // enable auto power on function
		Add10Min(&aTm);
		nHour = aTm.tm_hour;
		nMinute = aTm.tm_min;
		FDAutoPower_SetDateTime(nDay, nHour, nMinute, nSecond);
		printf("please shutdown your computer, and then it automatically power on after 10 minutes.\n");
	}
	else {
		FDAutoPower_Disable();
	}
}

/************************************************************************
		External SMI
************************************************************************/
BOOL gGotExtSMI = FALSE;

void OnExtSmiEvent(void)
{
	gGotExtSMI = TRUE;
	printf("got External SMI event\n");
}

void ExtSmiExample()
{
	// the board supports external SMI or not
	if ( !FDSdk_SupportExternalSMI() ) {
		printf("The board does not support this function\n");
		return;
	}

	printf("When got external SMI, close the demo.\n");

	// set call back function
	FDExtSmi_SetCallbackEvent(OnExtSmiEvent);

	// start external SMI
	FDExtSmi_Start();

	// wait to get external SMI event
	while ( !gGotExtSMI ) {
		VIA_Sleep(1);
		printf("wait to get external SMI\n");
	}

	// stop external SMI
	FDExtSmi_Stop();
}

/************************************************************************
  SDK
************************************************************************/
static void ShowFDItems(void)
{
	printf("\nFliteDeck example\n"
		"1: GPIO\n"
		"2: SMBUS\n"
		"3: LCD backlight control\n"
		"4: watchdog timer\n"
		"5: hardware monitor\n"
		"6: auto power on\n"
		"7: external SMI\n"
		"0: exit\n ");
}

void GBS_Example(void)
{
	int nSel;
	do {
		ShowFDItems();
		scanf("%d", &nSel);
		switch ( nSel) {
			case 1: GpioExample(); break;
			case 2: SmbusExample(); break;
			case 3: LcdBltExample(); break;
			case 4: WdtExample(); break;
			case 5: HwmExample(); break;
			case 6: AutoPowerExample(TRUE); break;
			case 7: ExtSmiExample(); break;
			default: return;
		}
	} while ( 1 );
}

void SdkExample()
{
	char szBoardName[80];
	DWORD nRetCode = FD_Initialize(szBoardName);
	if ( FD_S_OK!=nRetCode ) {
		switch ( nRetCode ) {
		case FD_E_ALREADY_INITIALIZED:
			printf("FliteDeck SDK is already initialized, it cannot be opened twice.\n"); break;
		case FD_E_BOARD_NOT_SUPPORT:
			printf("Not support the board.\n"); break;
		default:
			printf("Open driver error.\n"); break;
		}
		return;
	}
	printf("board name = %s\n", szBoardName);

	GBS_Example();

	// You MUST BE SURE that FD_UnInitialize() is called when the application is closed.
	// Otherwise FD_Initialize() cannot be normally called next time.
	FD_UnInitialize();
}

int main()
{
	SdkExample();
}
